package de.intevation.flys.client.server;

import java.util.ArrayList;
import java.util.HashMap;

import org.apache.log4j.Logger;

import de.intevation.flys.client.shared.exceptions.ServerException;
import de.intevation.flys.client.shared.model.Artifact;
import de.intevation.flys.client.shared.model.Collection;
import de.intevation.flys.client.shared.model.Recommendation;

import de.intevation.flys.client.client.services.LoadArtifactService;

/**
 * This service creates a new Artifact based on a given Recommendation and puts
 * this new artifact into a specified Collection.
 *
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class LoadArtifactServiceImpl
extends      ArtifactServiceImpl
implements   LoadArtifactService
{
    private static final Logger logger =
        Logger.getLogger(LoadArtifactServiceImpl.class);


    public static final String ERROR_LOAD_ARTIFACT = "error_load_artifact";


    /**
     * Clones or creates a single artifact and adds it to a collection.
     *
     * Note that in contrast to loadMany, always the given factory is used
     * to clone the artifact.
     *
     * @param parent  collection to add recommendation to.
     * @param recom   recommendation to create clone for.
     * @param factory factory to use.
     * @param url     servers url.
     * @param locale  the locale to translate messages.
     */
    public Artifact load(
        Collection     parent,
        Recommendation recom,
        String         factory,
        String         url,
        String         locale
    )
    throws ServerException {
        logger.info(
            "LoadArtifactServiceImpl.load: " + recom.getMasterArtifact());

        // 1) Clone the Artifact specified in >>recom<<
        Artifact clone = ArtifactHelper.createArtifact(
            url, locale, factory, recom);

        if (clone != null) {
            logger.debug("Successfully create Artifact Clone. Add now!");
            Collection c = CollectionHelper.addArtifact(
                parent, clone, url, locale);

            if (c != null) {
                logger.debug("Successfully added Clone to Collection.");

                return clone;
            }
        }

        throw new ServerException(ERROR_LOAD_ARTIFACT);
    }


    /**
     * Clone/create one or more artifacts and add it to a collection, avoiding
     * duplicates.
     *
     * @param parent  Collection where clones will be added to.
     * @param recoms  definitions of source of clone.
     * @param factory name of factory to use when cloning artifacts (can be
     *                null in which case the recommendations getFactory() will
     *                be used.
     * @param url     servers url.
     * @param locale  the locale to translate messages.
     *
     * @return cloned artifacts (same artifact might be contained multiple
     *         times).
     */
    public Artifact[] loadMany(
        Collection       parent,
        Recommendation[] recoms,
        String           factory,
        String           url,
        String           locale
    )
    throws ServerException {
        logger.debug("LoadArtifactServiceImpl.loadMany");

        ArrayList<Artifact> artifacts = new ArrayList<Artifact>();
        HashMap<Recommendation, Artifact> cloneMap =
            new HashMap<Recommendation, Artifact>();

        // TODO Respect the index of what to clone.

        // 1) Clone the Artifacts specified in >>recoms<<
        for (Recommendation recom : recoms) {
            // Do not do two clones of two identical recommendations.
            Artifact prevClone = cloneMap.get(recom);
            if (prevClone != null) {
                // Already cloned a recommendation like this.
                logger.debug("LoadArtifactServiceImpl: Avoid reclones, "
                    + "clone already exists.");
                artifacts.add(prevClone);
            }
            else {
                // Not already cloned.
                String realFactory = factory != null
                    ? factory
                    : recom.getFactory();

                logger.debug("One will be cloned with : " + realFactory);

                Artifact clone = ArtifactHelper.createArtifact(
                    url, locale, realFactory, recom);
        
                if (clone != null) {
                    logger.debug("LoadArtifactServiceImple: Successfully "
                        + "loaded Artifact Clone.");
                    Collection c = CollectionHelper.addArtifact(
                        parent, clone, url, locale);
        
                    if (c != null) {
                        artifacts.add(clone);
                        // Remember we cloned a recommendation like this.
                        cloneMap.put(recom, clone);
                    }
                    else {
                        throw new ServerException(ERROR_LOAD_ARTIFACT);
                    }
                }
            }
        }
        return artifacts.toArray(new Artifact[artifacts.size()]);
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
